{*****************************************}
{* Main unit for LogicGates8 application *}
{*****************************************}

// Implementation of the interactive graphics:
//   - The circuit layout is constituted by a mixture of shapes and images.
//   - The LEDs are colored ellipses; different color styles are used for on/off status.
//   - The movement of the switches is done by changing the vertical position (TOP property) of the corresponding shapes.
//   - the switch shapes being very small and difficult to click by the user, I used a transperant image, placed over them;
//     the OnClick event of the image is used to trigger the actions to be done when the corresponding switch has been clicked.

unit incdec;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus, StdCtrls, ExtCtrls,
  adder4f, adder4fct, subtractor4f, addsub4f, common;

type
  {**********}
  { TfIncDec }
  {**********}
  TfIncDec = class(TForm)
    mMenu: TMainMenu;
    mFile, mFileExit: TMenuItem;
    mHelp, mHelpAbout: TMenuItem;
    StaticText1: TStaticText;
    Memo1, Memo2, Memo3, Memo4: TMemo;
    btSubtractor1h: TButton;
    btSubtractor1f: TButton;
    btSubtractor4f: TButton;
    btAddSub4f: TButton;
    procedure mFileExitClick(Sender: TObject);
    procedure mHelpAboutClick(Sender: TObject);
    procedure btSubtractor1hClick(Sender: TObject);
    procedure btSubtractor1fClick(Sender: TObject);
    procedure btSubtractor4fClick(Sender: TObject);
    procedure btAddSub4fClick(Sender: TObject);
  end;

var
  fIncDec: TfIncDec;

implementation

{$R *.lfm}

{**********}
{ TfIncDec }
{**********}

{ Menu item "File > Exit": Exit application }

procedure TfIncDec.mFileExitClick(Sender: TObject);

begin
  Close;
end;

{ Menu item "Help > About": Display application about }

procedure TfIncDec.mHelpAboutClick(Sender: TObject);

var
  S: string;

begin
  S := 'Electronics simulation.' + LineEnding;
  S += 'Using logic gates to make binary incrementer and decrementer circuits.' + LineEnding + LineEnding;
  S += 'Version 1.0, © allu, May - September 2021.';
  MessageDlg('About "LogicGates8"', S, mtInformation, [mbOK], 0);
end;

{ Button "4-bit incrementer": Run 4-bit incrementer simulation }

procedure TfIncDec.btSubtractor1hClick(Sender: TObject);

begin
  fAdder4f.ShowModal;
end;

{ Button "4-bit incrementer (contraction)": Run 4-bit incrementer (contraction) simulation }

procedure TfIncDec.btSubtractor1fClick(Sender: TObject);

begin
  fAdder4fCt.ShowModal;
end;

{ Button "4-bit decrementer": Run 4-bit decrementer simulation }

procedure TfIncDec.btSubtractor4fClick(Sender: TObject);

begin
  fSubtractor4f.ShowModal;
end;

{ Button "4-bit incrementer-decrementer": Run 4-bit incrementer-decrementer simulation }

procedure TfIncDec.btAddSub4fClick(Sender: TObject);

begin
  fAddSub4f.ShowModal;
end;

end.

