{******************************************************************************}
{* 4-bit full-subtractor (4-bit decrementer) unit for LogicGates8 application *}
{******************************************************************************}

unit subtractor4f;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Grids, common;

type
  {****************}
  { TfSubtractor4f }
  {****************}
  TfSubtractor4f = class(TForm)
    Shape1: TShape;
    StaticText1: TStaticText;
    Label1, Label2, Label8, Label18, Label9, Label11, Label12, Label3, Label4: TLabel;
    laA0, laA1, laA2, laA3: TLabel;
    laB0, laB1, laB2, laB3: TLabel;
    laS0, laS1, laS2, laS3: TLabel;
    Shape20, Shape21, Shape22, Shape23, Shape28: TShape;
    Shape45, Shape50, Shape51: TShape;
    Shape52, Shape53, Shape54, Shape57, Shape60, Shape61: TShape;
    Shape62, Shape64, Shape65, Shape66, Shape67, Shape68, Shape69: TShape;
    Shape70, Shape71, Shape72, Shape73, Shape74, Shape75, Shape76, Shape77: TShape;
    Shape79, Shape80, Shape81, Shape82, Shape83, Shape85: TShape;
    Shape86, Shape88, Shape89, Shape90, Shape91, Shape92, Shape93: TShape;
    Shape95, Shape96, Shape97, Shape117, Shape118, Shape119, Shape120, Shape121: TShape;
    Shape100, Shape101, Shape102, Shape103, Shape104, Shape105, Shape106: TShape;
    Shape107, Shape108, Shape109, Shape110, Shape111, Shape112, Shape114: TShape;
    Shape122, Shape123, Shape124, Shape125, Shape126, Shape127, Shape128: TShape;
    Shape129, Shape25, Shape32, Shape36, Shape38, Shape55, Shape58, Shape59: TShape;
    Shape78, Shape87, Shape94, Shape98, Shape99: TShape;
    Image1, Image2, Image3, Image4, Image5, Image6, Image7, Image8, Image9: TImage;
    imSwitchA0, imSwitchA1, imSwitchA2, imSwitchA3: TImage;
    shSwitchA0a, shSwitchA0b, shSwitchA1a, shSwitchA1b: TShape;
    shSwitchA2a, shSwitchA2b, shSwitchA3a, shSwitchA3b: TShape;
    shLedA0, shLedA1, shLedA2, shLedA3: TShape;
    shLedB0, shLedB1, shLedB2, shLedB3: TShape;
    shLedS0, shLedS1, shLedS2, shLedS3, shLedCout: TShape;
    sgSub: TStringGrid;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure imSwitchA0Click(Sender: TObject);
    procedure imSwitchA1Click(Sender: TObject);
    procedure imSwitchA2Click(Sender: TObject);
    procedure imSwitchA3Click(Sender: TObject);
  private
    aLeds: TLeds;
    aSwitches: TSwitches;
    aInput, aOutput: TStatus;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape)
  DSwitches: array[0..3] of TSwitchData = (
    (Name: 'SwitchA0';
     Data: (512, 534, 514)),
    (Name: 'SwitchA1';
     Data: (378, 400, 380)),
    (Name: 'SwitchA2';
     Data: (246, 268, 248)),
    (Name: 'SwitchA3';
     Data: (112, 134, 114))
  );

var
  fSubtractor4f: TfSubtractor4f;

implementation

{$R *.lfm}

{ 4-bit-full-subtractor: Set switch position; do binary subtraction; turn LEDs on/off; fill-in stringgrid }

procedure SwitchAndAdd(DSwitches: array of TSwitchData; Switches: TSwitches; var InputStatus: TStatus;
  Leds: TLeds; var OutputStatus: TStatus; IX: Integer);

var
  Exp, I: Integer;
  A, B, C, Sout, Cout, S1, S2, C1, C2: Boolean;
  Dec: array[0..3] of Integer;
  Bin, Compl: TBinary;

begin
  // Change current input from 0 to Vs (and vice-versa)
  InputStatus[IX] := not InputStatus[IX];
  // Corresponding LED status
  OutputStatus[IX] := InputStatus[IX];
  // Set switch positions
  SwitchesOnOff(DSwitches, Switches, InputStatus);
  // Do the subtraction, using 4 1-bit-full-adders, adding A and the 2's complement of B, as is done in a real circuit
  Cout := True;                                                                // Cin of first adder set to +Vs (in order to form 2's complement)
  for I := 0 to 3 do begin
    A := OutputStatus[I];                                                      // actual bit for A input
    B := not OutputStatus[I + 4];                                              // actual bit for B input (inverted in order to form 1's complement)
    C := Cout;                                                                 // Cin = previous Cout
    // 1-bit full-adder binary addition
    Adder(A, B, C, S1, C1, S2, C2, Sout, Cout);
    // Actual bit sum LED status
    OutputStatus[I + 8] := SOut;
  end;
  // For a binary subtraction, Cout of the last adder may be used to indicate the polarity of the subtraction result
  OutputStatus[12] := Cout;
  // Turn all LEDs on/off
  LedsOnOff(Leds, OutputStatus);
  // Fill binary values into stringgrid
  for I := 0 to 3 do begin
    fSubtractor4f.sgSub.Cells[4 - I, 1] := ' ' + IntToStr(Bits[OutputStatus[I]]);
    fSubtractor4f.sgSub.Cells[4 - I, 2] := ' ' + IntToStr(Bits[OutputStatus[I + 4]]);
    fSubtractor4f.sgSub.Cells[4 - I, 3] := ' ' + IntToStr(Bits[OutputStatus[I + 8]]);
    Bin[3 - I] := OutputStatus[I + 8];
  end;
  // Calculate and fill in result's 2nd complement
  Compl := Complement2(Bin);
  for I := 0 to 3 do
    fSubtractor4f.sgSub.Cells[I + 1, 4] := ' ' + IntToStr(Bits[Compl[I]]);
  // Calculate decimal values
  Dec[0] := 0; Dec[1] := 0; Dec[2] := 0; Exp := 1;
  for I := 0 to 3 do begin
    Dec[0] += Bits[OutputStatus[I]] * Exp; Dec[1] += Bits[OutputStatus[I + 4]] * Exp; Dec[2] += Bits[OutputStatus[I + 8]] * Exp;
    Exp *= 2;
  end;
  // Decimal result may be positive or negative
  if Dec[0] < Dec[1] then
    Dec[2] := -(16 - Dec[2]);
  // Fill-in decimal values
  for I := 0 to 2 do begin
    fSubtractor4f.sgSub.Cells[5, I + 1] := ' ' + IntToStr(Dec[I]);
    if Abs(Dec[I]) < 10 then
      fSubtractor4f.sgSub.Cells[5, I + 1] := ' ' + fSubtractor4f.sgSub.Cells[5, I + 1];
    if Dec[I] >= 0 then
      fSubtractor4f.sgSub.Cells[5, I + 1] := ' ' + fSubtractor4f.sgSub.Cells[5, I + 1];
  end;
end;

{****************}
{ TfSubtractor4f }
{****************}

{ Application start: Initialisation }

procedure TfSubtractor4f.FormCreate(Sender: TObject);

begin
  // Arrays with switches (transparent image and bar shapes) and their "on/off" status
  SetLength(aSwitches, 4); SetLength(aInput, 8);
  aSwitches[0].Switch0 := imSwitchA0; aSwitches[0].Switch1 := shSwitchA0a; aSwitches[0].Switch2 := shSwitchA0b;
  aSwitches[1].Switch0 := imSwitchA1; aSwitches[1].Switch1 := shSwitchA1a; aSwitches[1].Switch2 := shSwitchA1b;
  aSwitches[2].Switch0 := imSwitchA2; aSwitches[2].Switch1 := shSwitchA2a; aSwitches[2].Switch2 := shSwitchA2b;
  aSwitches[3].Switch0 := imSwitchA3; aSwitches[3].Switch1 := shSwitchA3a; aSwitches[3].Switch2 := shSwitchA3b;
  // Arrays with LEDs (shape) and their "on/off" status
  SetLength(aLeds, 13); SetLength(aOutput, 13);
  aLeds[0] := shLedA0; aLeds[1] := shLedA1; aLeds[2] := shLedA2; aLeds[3] := shLedA3;
  aLeds[4] := shLedB0; aLeds[5] := shLedB1; aLeds[6] := shLedB2; aLeds[7] := shLedB3;
  aLeds[8] := shLedS0; aLeds[9] := shLedS1; aLeds[10] := shLedS2; aLeds[11] := shLedS3; aLeds[12] := shLedCout;
  // Use subscripts in input/output labels
  laA0.Caption := 'A' + SUB_Digits[0]; laA1.Caption := 'A' + SUB_Digits[1];
  laA2.Caption := 'A' + SUB_Digits[2]; laA3.Caption := 'A' + SUB_Digits[3];
  laB0.Caption := 'B' + SUB_Digits[0]; laB1.Caption := 'B' + SUB_Digits[1];
  laB2.Caption := 'B' + SUB_Digits[2]; laB3.Caption := 'B' + SUB_Digits[3];
  laS0.Caption := 'S' + SUB_Digits[0]; laS1.Caption := 'S' + SUB_Digits[1];
  laS2.Caption := 'S' + SUB_Digits[2]; laS3.Caption := 'S' + SUB_Digits[3];
end;

{ Form show-up: Initialize 4-bit decrementer simulation }

procedure TfSubtractor4f.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All inputs for A, the number to be decremented, to logic 0
  // For B, the decrement, least significant bit to logic 1 (decrement = subtraction of 0001)
  for I := 0 to Length(aInput) - 1 do begin
    if I = 4 then
      aInput[I] := True
    else
      aInput[I] := False;
  end;
  SwitchesOnOff(DSwitches, aSwitches, aInput);
  // All A outputs to logic 0, B0 output (least significant bit of increment) to logic 1,
  // all S outputs to logic 1 and Cout to logic 0 (0 - 1 = -1, which is a negative number)
  for I := 0 to Length(aOutput) - 1 do begin
    if (I = 4) or ((I >= 8) and (I <> 12)) then
      aOutput[I] := True
    else
      aOutput[I] := False;
  end;
  LedsOnOff(aLeds, aOutput);
end;

{ Switch A0, A1, A2, A3 activated: Do 4-bit decrementer simulation for actual switch positions }

procedure TfSubtractor4f.imSwitchA0Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, aInput, aLeds, aOutput, 0);               // add 2's complement of 0001 to 4-bit binary A
end;

procedure TfSubtractor4f.imSwitchA1Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, aInput, aLeds, aOutput, 1);
end;

procedure TfSubtractor4f.imSwitchA2Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, aInput, aLeds, aOutput, 2);
end;

procedure TfSubtractor4f.imSwitchA3Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, aInput, aLeds, aOutput, 3);
end;

{ Button "Close": Close the window }

procedure TfSubtractor4f.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

